import { Component, OnInit } from '@angular/core';

// type
// text email checkbox radio password select

interface FormItem{
  label: string;
  value: string;
  type: string;
  tips: string;
}
export class FormBase{
  label: string = '';
  type: string = '';
  id: number = new Date().getTime()
  constructor(type: string,label: string){
    this.type = type;
    this.label = label;
  }
}
export class FormText extends FormBase{
  value: string = '';
  constructor(label: string){
    super('text',label)
  }
}
export class FormUrl extends FormBase{
  value: string = '';
  constructor(label: string){
    super('url',label)
  }
}
export class FormTel extends FormBase{
  value: number;
  constructor(label: string){
    super('tel',label)
  }
}
export class FormNumber extends FormBase{
  value: number;
  constructor(label: string){
    super('number',label)
  }
}
export class FormPassword extends FormBase{
  value: string = '';
  constructor(label: string){
    super('password',label)
  }
}

export class FormColor extends FormBase{
  value: string = '';
  constructor(label: string){
    super('color',label)
  }
}

export class FormDatetime extends FormBase{
  value: string = new Date().toString();
  constructor(label: string){
    super('datetime-local',label)
  }
}
export class FormTime extends FormBase{
  value: string = new Date().toString();
  constructor(label: string){
    super('time',label)
  }
}
export class FormDate extends FormBase{
  value: string = new Date().toString();
  constructor(label: string){
    super('date',label)
  }
}

export class FormMonth extends FormBase{
  value: string = new Date().toString();
  constructor(label: string){
    super('month',label)
  }
}

export class FormWeek extends FormBase{
  value: string = new Date().toString();
  constructor(label: string){
    super('week',label)
  }
}

export class FormSearch extends FormBase{
  list: any[];
  value: string = '';
  constructor(label: string,list: any[]){
    super('search',label)
    this.list = list;
  }
}

export class FormSelect extends FormBase{
  list: any[];
  value: string = '';
  constructor(label: string,list: any[]){
    super('select',label)
    this.list = list;
  }
}

export class FormCheckboxItem{
  title: string = '';
  value: string = '';
  id: number = new Date().getTime()

  constructor(label: string,value: string){
    this.title = label;
    this.value = value;
  }
}

class FormCheckbox extends FormBase{
  list: FormCheckboxItem[]
  constructor(label: string,list: any[]){
    super('checkbox',label)
    this.list = list;
  }

  addItem(label: string,value: string){
    let item = new FormCheckboxItem(label,value);
    this.list.push(item)
  }

  delItem(index: number){
    this.list.splice(index,1)
  }

  editItem(item){
    this.list.map(res=>{
      if(item.id == res.id){
        res = item;
      }
    })
  }
}

export class FormRadio extends FormBase{
  list:any[];
  constructor(label: string,list: any[]){
    super('radio',label)
    this.list = list;
  }

  addItem(label: string,value: string){
    let item = new FormCheckboxItem(label,value);
    this.list.push(item)
  }

  delItem(index: number){
    this.list.splice(index,1)
  }

  editItem(item){
    this.list.map(res=>{
      if(item.id == res.id){
        res = item;
      }
    })
  }
}

@Component({
  selector: 'bts-forms',
  templateUrl: './forms.component.html',
  styleUrls: ['./forms.component.scss']
})
export class BtsFormsComponent implements OnInit {
  items: any[];
  constructor() {
    this.items = []
  }

  checkText(type?: string){
    if(
        type == 'text' ||
        type == 'color' ||
        type == 'url' ||
        type == 'tel' ||
        type == 'password' ||
        type == 'number' ||
        type == 'datetime-local' ||
        type == 'date' ||
        type == 'time' ||
        type == 'week' ||
        type == 'month'
    ){
      return true;
    }else{
      return false;
    }
  }

  createText(label: string){
    let item = new FormText(label)
    this.items.push(item)
  }

  createColor(label: string){
    let item =new FormColor(label)
    this.items.push(item)
  }

  createNumber(label: string){
    let item = new FormNumber(label)
    this.items.push(item)
  }

  createTel(label){
    let item = new FormTel(label)
    this.items.push(item)
  }

  createPassword(label: string){
    let item = new FormPassword(label)
    this.items.push(item)
  }

  createUrl(label: string){
    let item = new FormUrl(label)
    this.items.push(item)
  }

  createDatetime(label: string){
    let item = new FormDatetime(label)
    this.items.push(item)
  }

  createDate(label: string){
    let item = new FormDate(label)
    this.items.push(item)
  }

  createTime(label: string){
    let item = new FormTime(label)
    this.items.push(item)
  }

  createMonth(label: string){
    let item = new FormMonth(label)
    this.items.push(item)
  }

  createWeek(label: string){
    let item = new FormWeek(label)
    this.items.push(item)
  }
  createSearch(label: string,list: any[]){
    let item = new FormSearch(label,list)
    this.items.push(item)
  }

  createSelect(label:string,list: any[]){
    let item = new FormSelect(label,list)
    this.items.push(item)
  }

  createCheckbox(label: string, list: any[]){
    let item = new FormCheckbox(label,list)
    this.items.push(item)
  }

  createRadio(label: string, list: any[]){
    let item = new FormRadio(label,list)
    this.items.push(item)
  }

  ngOnInit() {
  }

}
